-- vzTetris by vonzippenstein. 29 AUG 2014
-- This is my first Love2D game and a learning experience.
-- Started with something I knew I could accomplish in not a whole lot of time.
-- Feel free to do whatever you like with it. It's yours now. Enjoy!
function love.load()
	-- The amazing music. Artist: Beldoroon
	-- http://modarchive.org/module.php?35781
	music = love.audio.newSource("bd_ttr1.xm")
	music:setVolume(1)
	musicOn = 1
	
	-- Load sound effects. Played at half volume.
	blockLand = love.audio.newSource("Blockland.wav")
	blockLand:setVolume(0.5)
	blockBreak = love.audio.newSource("Blockbreak.wav")
	blockBreak:setVolume(0.5)
	levelUp = love.audio.newSource("Levelup.wav")
	levelUp:setVolume(0.5)
	gameOverSound = love.audio.newSource("Gameover.wav")
	gameOverSound:setVolume(0.5)
	gamePauseSound = love.audio.newSource("Pause.wav")
	gamePauseSound:setVolume(0.5)
	
	-- Add the font. It's an altered version of Karmatic Arcade.
	-- Dropped the shadowing due to the font system adding a 1 pixel spaces between letters.
	font = love.graphics.newImageFont("KarmaArc.png", " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.:!?-/")
	font_default = love.graphics.newFont(12)
	
	-- Set the background color to black.
	love.graphics.setBackgroundColor(0, 0, 0)
	
	-- Create the game board.
	stage = {grid =  {{}}, 10, 21}
		
	-- Create shapes.
	shape = {{{0,0,0,0},
					{0,1,1,0},
					{0,1,1,0},
					{0,0,0,0}} ,
				  {{0,0,0,0},
					{2,2,2,2},
					{0,0,0,0},
					{0,0,0,0}},
				  {{0,0,0,0},
					{0,0,3,3},
					{0,3,3,0},
					{0,0,0,0}},
				  {{0,0,0,0},
					{0,4,4,0},
					{0,0,4,4},
					{0,0,0,0}},
				  {{0,0,0,0},
					{0,5,5,5},
					{0,5,0,0},
					{0,0,0,0}}, 
				  {{0,0,0,0},
					{0,6,6,6},
					{0,0,0,6},
					{0,0,0,0}},
				  {{0,0,0,0},
					{0,7,7,7},
					{0,0,7,0},
					{0,0,0,0}}}
					
	-- Shuffle the deck.
	math.randomseed(os.time())
	
	-- Initial players and scores for the board.
	names = {"VZE","ABC","DEF","GHI","JKL","MNO","PQR","STU","VWX","YNZ","END"}
	scores = {1000,900,800,700,600,500,400,300,200,100,0}
	-- These two arrays have to do with an over complicated solution to a problem with text alignment.
	adjust = {1,0,0,0,0,0,0,0,0,1,0} -- This is the number of pixels to shift the text right.
	number = {0,0,0,0,0,0,0} -- And this is a 7 digit number broken into a single digits.
	
	-- Load highscores.
	loadHighscores()
	
	-- Start game.
	reset()
end

function reset() -- Clears current game and set's up a new game.
	if gameOver == 1 then
		-- If this isn't the first game then add player score, sort, and save all scores.
		if score > scores[10] then 
			names[10] = name
			scores[10] = score
		end
		sortHighscore()
		saveHighscores()
	end
	
	-- Reset the soundtrack.
	if musicOn == 1 then 
		love.audio.stop(music)
		love.audio.play(music)
	end
	
	-- Blank game stage.
	stage.grid = {{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0},
						{0,0,0,0,0,0,0,0,0,0}}
	-- Initialize game variables.
	gamePause = 0	-- If the game is paused.
	gameOver = 0 -- If the game is over.
	gameNotPause = 1 -- If the game isn't paused. Wait what? It's an unpause timer.
	gameTime = 0 -- Starts at game start. Stops at game end.
	stageShift = 0 --Tracks when the game last forced a piece to shift down.
	stageLevel = 1 -- Game speed increases with this number and interface color.
	controlTimer = 0 -- Adds a delay between button input for better control.
	rowsComplete = 0 -- 10 rows per level
	songTimer = 0 -- Counts how long has been playing for the loop.
	score = 0 -- My favorite of Beavis and Butthead's catchphrases.
	blink = 0 -- BLEEDING EDGE BLINK COUNTING TECH
	blackout = 0 -- Counter for the screen dim.
	name = "???" -- WHAT'S YOUR NAME???
	letter = 1 -- Use to select which letter to change when entering a highscore.
	bNext = 0 -- Upcoming block displayed on the top left.
	b = 0 -- Which block out of the seven shapes is on the board. 0 is a dead block.
	o = 1 -- How the block is rotated. Why O? Because it's a circle.
	x = 6 -- http://www.youtube.com/watch?v=fGx6K90TmCI
	y = 2 -- http://www.youtube.com/watch?v=aW8UnXzP3ms
end

function love.update(dt)
	-- Where the magic happens.
	controlTimer = controlTimer - dt -- Cooldown timer between input actions.
	if controlTimer <= 0 then keydown() end -- If nicely chilled then perform button held functions.
	if gamePause == 1 or gameOver == 1 then -- Exit the update if paused or game is over.
		gameNotPause = 0 -- Not pause is a timer which holds the game for a second after unpausing.
		return -- Cut the function short.
	else
		if gameNotPause < 1 then gameNotPause = gameNotPause + dt end 
		-- Loops the music. 142 is the length.
		if songTimer >= 142 then
			love.audio.stop(music)
			love.audio.play(music)
			songTimer = 0
		else
			if musicOn == 1 then songTimer = songTimer + dt end
		end
	end
	if gameNotPause < 1 or gameOver == 1 then return end --No updates for game over or until unpause counts down.
	gameTime = gameTime + dt --Time since start.
	--20 levels max. Anymore and the speed is unplayable.
	if stageLevel < 21 then stageLevel = math.floor(rowsComplete/10) + 1 end
	stageTimer = 0.26 - (stageLevel/100) --Fasten your seatbelt.
	if gameTime - stageShift > stageTimer then --Board timer.
		stageShift = gameTime --Reset counter
		if b == 0 then -- Block type is set to when it's done traveling.
			checkRows() -- See if it cleared any rows.
			if bNext == 0 then -- Add a new block and next block. (first run)
				b = roundBlock(math.random(0,7))
				bNext = roundBlock(math.random(0,7))
			else
				b = bNext -- Move next block to current block.
				bNext = roundBlock(math.random(0,7)) -- Add a new next block
			end
			o = 1 -- Reset rotation.
			x = 6 -- Reset position.
			y = 2
			createBlock(b) -- Drop it into the game world.
		else
			shiftBlock() -- Block is still traveling under player control.
		end
	end
end

function love.draw()
	--All the draws for the game.
	if blackout < 60 then --Screen dimmer.
		love.graphics.setColor(64,64,64) -- Grid color
		love.graphics.rectangle("fill", 103,8,113,223) -- Solid white fill.
		drawStage(94,12)
		drawInterface()
	end
end

function drawStage(stage_x,stage_y)
	-- O, I, S, Z, L, J, T
	-- 1, 2, 3, 4, 5, 6, 7
	for i=2, 21 do
		for j=1, 10 do
			-- I've been told that this is the most disgusting looking elseif in this planet's history.
			if stage.grid[i][j] == 1 then drawTealSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 2 then drawRedSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 3 then drawBlueSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 4 then drawGreenSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 5 then drawYellowSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 6 then drawPurpleSquare((11*j)+stage_x,(11*i)-stage_y)
			elseif stage.grid[i][j] == 7 then drawGreySquare((11*j)+stage_x,(11*i)-stage_y)
			else
				-- This block is empty, unembraced, a void endlessly aching, unconsoled.
				love.graphics.setColor(0,0,0)
				love.graphics.rectangle("fill",(11*j)+stage_x,(11*i)-stage_y,10,10)
			end
		end
	end
end

function drawInterface()
	--stage colors, pause blinker, and other foofoo sissy lala bull
	if stageLevel <= 3 then love.graphics.setColor(229,229,229) end
	if stageLevel >= 4  and stageLevel <= 6 then love.graphics.setColor(128,255,246) end
	if stageLevel >= 7  and stageLevel <= 9  then love.graphics.setColor(48,255,107) end
	if stageLevel >= 10  and stageLevel <= 12 then love.graphics.setColor(255,250,95) end
	if stageLevel >= 13  and stageLevel <= 15  then love.graphics.setColor(40,91,255) end
	if stageLevel >= 16  and stageLevel <= 18 then love.graphics.setColor(236,66,255) end
	if stageLevel >= 19  then love.graphics.setColor(255,50,50)end
	-- Blink used for the ??? on the scoreboard at game end and then pause.
	blink = blink + 1
	if blink >= 30 then
		blackout = blackout + 1
		blink = 0
	end
	
	-- Ready that text.
	love.graphics.setFont(font)
	-- Not sure if I like the pause display but it made the final cut.
	if gamePause == 1 and blink < 15 then
		love.graphics.print("-PAUSE-", 130, 116)
	end
	
	-- Right side of the interface.
	love.graphics.print("NEXT", 232, 22) -- Actual displayed block is at the end so it doesn't effect the text draw colors.
	love.graphics.print("LEVEL", 232, 82)
	if stageLevel < 10 then
		love.graphics.print(stageLevel, 252, 98)
	else 
		love.graphics.print(stageLevel, 247, 98)
	end
	love.graphics.print("LINES", 232, 121)
	if rowsComplete < 10 then -- If greater than shift character approach.
		love.graphics.print(rowsComplete, 252, 137)
	else
		if rowsComplete < 100 then
			love.graphics.print(rowsComplete, 247, 137)
		else
			love.graphics.print(rowsComplete, 242, 137)
		end
	end
	love.graphics.print("SCORE", 232, 159) 
	if score > 0 then -- Logarythmic shift approach.
		love.graphics.print(score, 257-(math.ceil(math.log10(score))*5),175)
	else
		love.graphics.print(score, 252, 175)
	end
	love.graphics.print("TIME", 232, 197)
	-- Displays how much of the player's life has been wasted playing video games.
	timeMinutes = math.floor(gameTime/60)
	timeSeconds = math.floor(gameTime)-(timeMinutes*60)
	if timeMinutes < 10 then tShift = 0 else tShift = -9 end
	love.graphics.print(timeMinutes, 243+tShift, 82+16+115)
	love.graphics.print(":", 253, 82+16+115) 
	if timeSeconds < 10 then -- Add an extra zero if less than 10
		love.graphics.print("0", 258, 82+16+115) 
		love.graphics.print(timeSeconds, 268, 213) 
	else
		love.graphics.print(timeSeconds, 258, 213) 
	end
	--Score board on the left side. This is the most recent addition.
	love.graphics.print("HIGHSCORE", 10, 22)
	p = 0 -- Pushes the scores down when the player makes the board.
	k = 0 -- Enables the display of the player score.
	for i=1, 10 do
		if score > scores[i] then
			p = p + 1
			k = 1
		end
		if i < 10 or k < 1 then 
			love.graphics.print(names[i], 10, 28+18*(i+k))
			if scores[i] < 1000 then -- Adjusts placement.
				love.graphics.print(scores[i], 40+20+adjust[i], 28+18*(i+k))
			elseif scores[i] < 10000 then -- Adjusts placement.
				love.graphics.print(scores[i], 40+10+adjust[i], 28+18*(i+k))
			else -- Adjusts placement.
				love.graphics.print(scores[i], 40+adjust[i], 28+18*(i+k))
			end
		end
	end
	if k > 0 then
		-- Color of the current player's score.
		love.graphics.setColor(229,229,229)
		-- Blink the ??? to draw attention if the player has made the board.
		if gameOver == 0 or name ~= "???" or blink < 15 then
			love.graphics.print(name, 10, 28+18*(11-p))
		end
		-- Places a square under the selected character when enterting a highscore.
		if gameOver == 1 then
			love.graphics.print("-", letter*10+2, 28+18*(11-p)+8)
		end
		if score < 1000 then -- Adjusts placement.
			love.graphics.print(score, 40+20+adjust[11], 28+18*(11-p))
		elseif score < 10000 then -- Adjusts placement.
			love.graphics.print(score, 40+10+adjust[11], 28+18*(11-p))
		else -- Adjusts placement.
			love.graphics.print(score, 40+adjust[11], 28+18*(11-p))
		end
	end
	-- O, I, S, Z, L, J, T
	-- 1, 2, 3, 4, 5, 6, 7
	if bNext < 3 then -- Aligns the O and I blocks for display.
		drawNextBox(bNext,208,32)
	else
		drawNextBox(bNext,218,32)
	end
end

function love.keypressed(k)
	--Perform the following actions when the player initially presses a key.
	blackout = 0 -- reset the screen dim timer.

	if k == "escape" then
		--Are you sure you want to quit?
		saveHighscores()
		os.exit()
	end
	
	if k == "return" then
		blink = 0 -- Sync the blink.
		if gameOver == 1 then
			reset() -- Wipes game and saves score. Hope you entered a name.
		else
			-- Everything on hold. Cut the music.
			if gamePause == 1 then
				gamePause = 0 
				love.audio.resume(music)
			else 
				gamePause = 1
				love.audio.pause(music)
			end
			love.audio.stop(gamePauseSound)
			love.audio.play(gamePauseSound)
		end
	end
	
	--A relic of an ancient build left to stand as a momument.
	if k == "m" then
		if musicOn == 1 then
			musicOn = 0
			love.audio.pause(music)
		else
			musicOn = 1
			love.audio.resume(music)
		end
	end
	
	if gameOver == 0 then
		--Paused do nothing. Kicks out.
		if gameNotPause < 1 then return end
		--Regular gameplay. Presses aren't bound by the control timer like hold.
		if k == "lctrl" or k == "lalt" or k == "tab" then
			rotateBlock()
		end
		
		if k == "right"  then
			moveBlockRight()
			controlTimer = 0.15
		end
		   
		if k == "left" then
			moveBlockLeft()
			controlTimer = 0.15
		end
		   
		if k == "down" then
			shiftBlock()
			controlTimer = 0.1 -- Faster!
		end
		
		-- Increase the speed of control needed for later levels.
		if k == "backspace" or k == "lshift" or k == " " then
			if controlTimer > 0.01 then
				controlTimer = 0.01
			end
		end
	else
		if score > scores[10]  then
			--Enter your initials.
			love.audio.stop(blockLand)
			love.audio.play(blockLand)
			-- Move the letter selector right.
			if k == "right" or k == "lctrl" or k == "lalt" or k == "tab" then
				if letter < 3 then letter = letter + 1 end
				controlTimer = 0.15 -- Not needed but couldn't hurt.
			end
			-- Move the letter selector left.
			 if  k == "left" or k == "backspace" or k == "lshift" or k == " " then
				if letter > 1 then letter = letter - 1 end
				controlTimer = 0.15 -- So then why not leave them in.
			end
			-- Next letter.
			if k == "up"  then
				cycleUp()
				controlTimer = 0.3
			end
			-- Previous Letter.
			if k == "down" then
				cycleDown()
				controlTimer = 0.3
			end
		end
	end
end

function keydown()
	-- This function cycles actions on the timer controlled by controlTimer.
	
	if gameOver == 1 and score > scores[10] then
		--Game is over and the player is entering initials.
		if love.keyboard.isDown("up") then
			love.audio.stop(blockLand)
			love.audio.play(blockLand)
			cycleUp()
			controlTimer = 0.1
		end
		if love.keyboard.isDown("down") then
			love.audio.stop(blockLand)
			love.audio.play(blockLand)
			cycleDown()
			controlTimer = 0.1
		end
	end
	
	-- Exit the rest of the game is paused.
	if gameNotPause < 1 then return end
	-- Holdin' that left button.
	if love.keyboard.isDown("left") then
		moveBlockLeft()
		controlTimer = 0.1
	end
	-- Holdin' that right button.
	if love.keyboard.isDown("right") then
		moveBlockRight()
		controlTimer = 0.1
	end
	-- Straight ahead one direction. Not left. Not right. More and more.
	if love.keyboard.isDown("down") then
		if b > 0 then shiftBlock() end
		controlTimer = 0.05 -- Twice as fast!
	end
	
	-- "America is all about speed. Hot, nasty, badass speed."
	if love.keyboard.isDown("backspace") or love.keyboard.isDown("lshift") or love.keyboard.isDown(" ") then
		if controlTimer > 0.01 then
			controlTimer = 0.01
		end
	end
end

function cycleUp()
	--Pull the selected character from name and cycle the char up skipping ones that don't exist in the font.
	rec = string.byte(name,letter)
	rec = rec + 1
	if rec == 34 then rec = 45 end
	if rec == 47 then rec = 48 end
	if rec == 59 then rec = 63 end
	if rec == 64 then rec = 65 end
	if rec > 90 then rec = 32 end
	if letter == 1 then name = string.char(rec,string.byte(name,2),string.byte(name,3)) end
	if letter == 2 then name = string.char(string.byte(name,1),rec,string.byte(name,3)) end
	if letter == 3 then name = string.char(string.byte(name,1),string.byte(name,2),rec) end
end

function cycleDown()
	--Pull the selected character from name and cycle the char down skipping ones that don't exist in the font.
	rec = string.byte(name,letter)
	rec = rec - 1
	if rec == 64 then rec = 63 end
	if rec == 62 then rec = 58 end
	if rec == 47 then rec = 46 end
	if rec == 44 then rec = 33 end
	if rec < 32 then rec = 90 end
	if letter == 1 then name = string.char(rec,string.byte(name,2),string.byte(name,3)) end
	if letter == 2 then name = string.char(string.byte(name,1),rec,string.byte(name,3)) end
	if letter == 3 then name = string.char(string.byte(name,1),string.byte(name,2),rec) end
end

function loadHighscores()
	-- Yep, it loads the scores.
	if love.filesystem.exists("highscores.txt") then
		file = love.filesystem.newFile("highscores.txt")
		file:open('r')
		for i = 1, 10 do
			data = file:read(3)
			names[i] = data
			data = file:read(7)
			scores[i] = data
			scores[i] = tonumber(scores[i])
		end
		file:close()
		score = 0
		adjustText()
	end
end

function saveHighscores()
	-- Yep, it saves the scores.
	file = love.filesystem.newFile("highscores.txt")
	file:open('w')
	for i=1, 10 do
		file:write(names[i])
		zeros = 100
		for j=1,4 do
			zeros = zeros*10
			if scores[i] < zeros then
				file:write("0")
			end
		end
		file:write(tostring(scores[i]))
	end
	file:close()
end


function sortHighscore()
	-- Ah list sort. This takes me back.
	for i = 1, 9 do
		if scores[11-i] > scores[10-i] then
			scores[11] = scores[10-i]
			names[11] = names[10-i]
			scores[10-i] = scores[11-i]
			names[10-i] = names[11-i]
			scores[11-i] = scores[11] 
			names[11-i] = names[11] 
		end
	end
	scores[11] = score --This probably doesn't do anything and doesn't need to be here so let's keep it.
	adjustText()
end

function adjustText()
	--Counts ones up to the millionth.
	--NOW WITHOUT STRINGS FOR MAXIMUM FUN
	--"Why is this needed?" Because the '1' font character is 1 pixel slimmer.
	--"This is stupid, who cares?" I care. That's who.
	for f = 1, 11 do
		adjust[f] = 0
		for z = 1, 7 do
			number[z] = scores[f] % 10^z
		end
		for z=7, 2, -1 do
			number[z] = (number[z] - number[z-1]) / 10^(z-1)
		end
		one_count = 0
		for z = 1, 7 do
			if number[z] == 1 then
				adjust[f] = adjust[f] + 1
			end
		end
	end
end

function roundBlock(bNumber)
	--"Why not just use the built in ceil fuction to round up?"
	--Originally I created this to tinker with the randomization.
	--Now I keep it around for sentimental value.
	if bNumber < 1 then
		return 1
	elseif bNumber < 2 then
		return 2
	elseif bNumber < 3 then
		return 3
	elseif bNumber < 4 then
		return 4
	elseif bNumber < 5 then
		return 5 
	elseif bNumber < 6 then
		return 6 
	else
		return 7 
	end
end

function createBlock(randomBlock)
	--Put a live block in the gameboard. If it instantly collides then the game is over.
	if randomBlock == 1 then
		playerType = 1
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 1 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 1 end
		if stage.grid[3][5] > 0 then gameOver = 1 else stage.grid[3][5] = 1 end
		if stage.grid[3][6] > 0 then gameOver = 1 else stage.grid[3][6] = 1 end
	elseif randomBlock == 2 then
		playerType = 2
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 2 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 2 end
		if stage.grid[2][4] > 0 then gameOver = 1 else stage.grid[2][4] = 2 end
		if stage.grid[2][7] > 0 then gameOver = 1 else stage.grid[2][7] = 2 end
	elseif randomBlock == 3 then
		playerType = 3
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 3 end
		if stage.grid[2][7] > 0 then gameOver = 1 else stage.grid[2][7] = 3 end
		if stage.grid[3][5] > 0 then gameOver = 1 else stage.grid[3][5] = 3 end
		if stage.grid[3][6] > 0 then gameOver = 1 else stage.grid[3][6] = 3 end
	elseif randomBlock == 4 then
		playerType = 4
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 4 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 4 end
		if stage.grid[3][6] > 0 then gameOver = 1 else stage.grid[3][6] = 4 end
		if stage.grid[3][7] > 0 then gameOver = 1 else stage.grid[3][7] = 4 end
	elseif randomBlock == 5 then
		playerType = 5
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 5 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 5 end
		if stage.grid[2][7] > 0 then gameOver = 1 else stage.grid[2][7] = 5 end
		if stage.grid[3][5] > 0 then gameOver = 1 else stage.grid[3][5] = 5 end
	elseif randomBlock == 6 then
		playerType = 6
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 6 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 6 end
		if stage.grid[2][7] > 0 then gameOver = 1 else stage.grid[2][7] = 6 end
		if stage.grid[3][7] > 0 then gameOver = 1 else stage.grid[3][7] = 6 end
	elseif randomBlock == 7 then
		playerType = 7
		if stage.grid[2][6] > 0 then gameOver = 1 else stage.grid[2][6] = 7 end
		if stage.grid[2][5] > 0 then gameOver = 1 else stage.grid[2][5] = 7 end
		if stage.grid[2][7] > 0 then gameOver = 1 else stage.grid[2][7] = 7 end
		if stage.grid[3][6] > 0 then gameOver = 1 else stage.grid[3][6] = 7 end
	end
	-- RIP
	if gameOver == 1 then
		love.audio.stop(gameOverSound)
		love.audio.play(gameOverSound)
		love.audio.stop(music)
		blink = 0 -- Sync the blink.
	end
end

function drawNextBox(theNextBlock,nextBlock_x,nextBlock_y)
	-- O, I, S, Z, L, J, T
	-- 1, 2, 3, 4, 5, 6, 7
	--Display the upcoming block with the appropriate shape and color.
	if theNextBlock == 1 then
		drawTealSquare((11*2)+nextBlock_x+15,(11*1)+nextBlock_y)
		drawTealSquare((11*3)+nextBlock_x+15,(11*1)+nextBlock_y)
		drawTealSquare((11*3)+nextBlock_x+15,(11*2)+nextBlock_y)
		drawTealSquare((11*2)+nextBlock_x+15,(11*2)+nextBlock_y)
	elseif theNextBlock == 2 then
		drawRedSquare((11*3)+nextBlock_x+15,(11*1)+nextBlock_y)
		drawRedSquare((11*2)+nextBlock_x+15,(11*1)+nextBlock_y)
		drawRedSquare((11*1)+nextBlock_x+15,(11*1)+nextBlock_y)
		drawRedSquare((11*4)+nextBlock_x+15,(11*1)+nextBlock_y)
	elseif theNextBlock == 3 then
		drawBlueSquare((11*3)+nextBlock_x,(11*1)+nextBlock_y)
		drawBlueSquare((11*4)+nextBlock_x,(11*1)+nextBlock_y)
		drawBlueSquare((11*2)+nextBlock_x,(11*2)+nextBlock_y)
		drawBlueSquare((11*3)+nextBlock_x,(11*2)+nextBlock_y)
	elseif theNextBlock == 4 then
		drawGreenSquare((11*3)+nextBlock_x,(11*1)+nextBlock_y)
		drawGreenSquare((11*2)+nextBlock_x,(11*1)+nextBlock_y)
		drawGreenSquare((11*3)+nextBlock_x,(11*2)+nextBlock_y)
		drawGreenSquare((11*4)+nextBlock_x,(11*2)+nextBlock_y)
	elseif theNextBlock == 5 then
		drawYellowSquare((11*3)+nextBlock_x,(11*1)+nextBlock_y)
		drawYellowSquare((11*2)+nextBlock_x,(11*1)+nextBlock_y)
		drawYellowSquare((11*4)+nextBlock_x,(11*1)+nextBlock_y)
		drawYellowSquare((11*2)+nextBlock_x,(11*2)+nextBlock_y)
	elseif theNextBlock == 6 then
		drawPurpleSquare((11*3)+nextBlock_x,(11*1)+nextBlock_y)
		drawPurpleSquare((11*2)+nextBlock_x,(11*1)+nextBlock_y)
		drawPurpleSquare((11*4)+nextBlock_x,(11*1)+nextBlock_y)
		drawPurpleSquare((11*4)+nextBlock_x,(11*2)+nextBlock_y)
	elseif theNextBlock == 7 then
		drawGreySquare((11*3)+nextBlock_x,(11*1)+nextBlock_y)
		drawGreySquare((11*2)+nextBlock_x,(11*1)+nextBlock_y)
		drawGreySquare((11*4)+nextBlock_x,(11*1)+nextBlock_y)
		drawGreySquare((11*3)+nextBlock_x,(11*2)+nextBlock_y)
	end
end

function shiftBlock()
	-- Checks collisions, adds lower blocks, and removes upper blocks shifting the piece downward.
	if b == 1 then
		if y < 20 and stage.grid[y+2][x] == 0 and stage.grid[y+2][x-1] == 0 then
			stage.grid[y+2][x] = b
			stage.grid[y+2][x-1] = b
			stage.grid[y][x] = 0
			stage.grid[y][x-1] = 0
			y = y + 1
		else
			b = 0
		end
	elseif b == 2 then
		if o == 1 then
			if y < 21 and stage.grid[y+1][x-2] == 0 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+1][x-2] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-2] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 19 and stage.grid[y+3][x] == 0 then
				stage.grid[y+3][x] = b
				stage.grid[y-1][x] = 0
				y = y + 1
			else
				b = 0
			end
		end
	elseif b == 3 then
		if o == 1 then
			if y < 20 and stage.grid[y+2][x-1] == 0 and stage.grid[y+2][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+2][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y+1][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 20 and stage.grid[y+1][x] == 0 and stage.grid[y+2][x+1] == 0 then
				stage.grid[y+1][x] = b
				stage.grid[y+2][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		end
	elseif b == 4 then
		if o == 1 then
			if y < 20 and stage.grid[y+1][x-1] == 0 and stage.grid[y+2][x] == 0 and stage.grid[y+2][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y+2][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 20 and stage.grid[y+2][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+2][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x] = 0
				stage.grid[y-1][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		end
	elseif b == 5 then
		if o == 1 then
			if y < 20 and stage.grid[y+2][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+2][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 20 and stage.grid[y+2][x] == 0 and stage.grid[y+2][x+1] == 0 then
				stage.grid[y+2][x] = b
				stage.grid[y+2][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y-1][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 4 then
			if y < 20 and stage.grid[y][x-1] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y-1][x] = 0
				y = y + 1
			else
				b = 0
			end
		end
	elseif b == 6 then
		if o == 1 then
			if y < 20 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+2][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+2][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 20 and stage.grid[y][x+1] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y][x+1] = b
				stage.grid[y+2][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y-1][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 4 then
			if y < 20 and stage.grid[y+2][x-1] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y+2][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x-1] = 0
				y = y + 1
			else
				b = 0
			end
		end
	elseif b == 7 then
		if o == 1 then
			if y < 20 and stage.grid[y+1][x-1] == 0 and stage.grid[y+2][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 2 then
			if y < 20 and stage.grid[y+1][x+1] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y+1][x+1] = b
				stage.grid[y+2][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y][x+1] = 0
				y = y + 1
			else
				b = 0
			end
		elseif o == 4 then
			if y < 20 and stage.grid[y+1][x-1] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+2][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x-1] = 0
				y = y + 1
			else
				b = 0
			end
		end
	end
	if b == 0 then
		love.audio.stop(blockLand)
		love.audio.play(blockLand)
	end
end

function moveBlockLeft()
	-- Checks collisions, adds left blocks, and removes right blocks shifting the piece to the left.
	if b == 1 then
		if x > 2 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x-2] == 0  then
			stage.grid[y][x-2] = b
			stage.grid[y+1][x-2] = b
			stage.grid[y][x] = 0
			stage.grid[y+1][x] = 0
			x = x - 1
		end
	elseif b == 2 then
		if o == 1 then
			if x > 3 and stage.grid[y][x-3] == 0  then
				stage.grid[y][x-3] = b
				stage.grid[y][x+1] = 0
				x = x - 1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 and stage.grid[y+2][x-1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y+2][x-1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				stage.grid[y+2][x] = 0
				x = x - 1
			end
		end
	elseif b == 3 then
		if o == 1 then
			if x > 2 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-2] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-2] = b
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x] = 0
				x = x -1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x+1] = 0
				x = x - 1
			end
		end	
	elseif b == 4 then
		if o == 1 then
			if x > 2 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y][x-2] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y][x] = 0
				stage.grid[y+1][x+1] = 0
				x = x - 1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		end	
	elseif b == 5 then 
		if o == 1 then
			if x > 2 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x-2] == 0 then
				stage.grid[y][x-2] = b
				stage.grid[y+1][x-2] = b
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x-1] = 0
				x = x - 1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x+1] = 0
				x = x - 1
			end
		elseif o == 3 then
			if x > 2 and stage.grid[y-1][x] == 0 and stage.grid[y][x-2] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y][x-2] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x+1] = 0
				x = x - 1
			end
		elseif o == 4 then
			if x > 2 and stage.grid[y-1][x-2] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x-2] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		end	
	elseif b == 6 then
		if o == 1 then
			if x > 2 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y][x-2] = b
				stage.grid[y+1][x] = b
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x+1] = 0
				x = x - 1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		elseif o == 3 then
			if x > 2 and stage.grid[y-1][x-2] == 0 and stage.grid[y][x-2] == 0 then
				stage.grid[y-1][x-2] = b
				stage.grid[y][x-2] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y][x+1] = 0
				x = x - 1
			end
		elseif o == 4 then
			if x > 2 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-2] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-2] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		end	
	elseif b == 7 then
		if o == 1 then
			if x > 2 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y][x-2] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		elseif o == 2 then
			if x > 1 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		elseif o == 3 then
			if x > 2 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-2] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-2] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x+1] = 0
				x = x - 1
			end
		elseif o == 4 then
			if x > 2 and stage.grid[y-1][x-1] == 0 and stage.grid[y][x-2] == 0 and stage.grid[y+1][x-1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-2] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x - 1
			end
		end	
	end
end

function moveBlockRight()
	-- Checks collisions, adds right blocks, and removes left blocks shifting the piece to the right.
	if b == 1 then
		if x < 10 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
			stage.grid[y][x+1] = b
			stage.grid[y+1][x+1] = b
			stage.grid[y][x-1] = 0
			stage.grid[y+1][x-1] = 0
			x = x + 1
		end
	elseif b == 2 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+2] == 0  then
				stage.grid[y][x+2] = b
				stage.grid[y][x-2] = 0
				x = x + 1
			end
		elseif o == 2 then
			if x < 10 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 and stage.grid[y+2][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y+2][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				stage.grid[y+2][x] = 0
				x = x + 1
			end
		end
	elseif b == 3 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x] = 0
				stage.grid[y+1][x-1] = 0
				x = x + 1
			end
		elseif o == 2 then
			if x < 9 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+2] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+2] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x+1] = 0
				x = x + 1
			end
		end	
	elseif b == 4 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+2] == 0 then
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+2] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		elseif o == 2 then
			if x < 9 and stage.grid[y-1][x+2] == 0 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+2] = b
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		end	
	elseif b == 5 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y][x+2] = b
				stage.grid[y+1][x] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x-1] = 0
				x = x + 1
			end
		elseif o == 2 then 
			if x < 9 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+2] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+2] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		elseif o == 3 then
			if x < 9 and stage.grid[y-1][x+2] == 0 and stage.grid[y][x+2] == 0 then
				stage.grid[y-1][x+2] = b
				stage.grid[y][x+2] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x-1] = 0
				x = x + 1
			end
		elseif o == 4 then
			if x < 10 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		end	
	elseif b == 6 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+2] == 0 then
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+2] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x+1] = 0
				x = x + 1
			end
		elseif o == 2 then
			if x < 9 and stage.grid[y-1][x+2] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+2] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		elseif o == 3 then
			if x < 9 and stage.grid[y-1][x] == 0 and stage.grid[y][x+2] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y][x+2] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y][x-1] = 0
				x = x + 1
			end
		elseif o == 4 then
			if x < 10 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x-1] = 0
				x = x + 1
			end
		end	
	elseif b == 7 then
		if o == 1 then
			if x < 9 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		elseif o == 2 then
			if x < 9 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+2] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+2] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		elseif o == 3 then
			if x < 9 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+2] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+2] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x-1] = 0
				x = x + 1
			end
		elseif o == 4 then
			if x < 10 and stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x] = 0
				x = x + 1
			end
		end	
	end
end

function rotateBlock()
	-- Checks collisions, adds new rotation blocks, and removes old rotation blocks.
	-- O, I, S, Z, L, J, T
	-- 1, 2, 3, 4, 5, 6, 7
	if b == 2 then
		if o == 1 then
			if y < 20 and stage.grid[y-1][x] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+2][x] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y+1][x] = b
				stage.grid[y+2][x] = b
				stage.grid[y][x-2] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y][x+1] = 0
				o = 2
			end
		elseif o == 2 then
			if x <= 2 then
				moveBlockRight()
				moveBlockRight()
			elseif x == 10 then
				moveBlockLeft()
			end
			if x > 2 and x < 10 and y < 20 and stage.grid[y][x-2] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y][x+1] == 0 then
				stage.grid[y][x-2] = b
				stage.grid[y][x-1] = b
				stage.grid[y][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x] = 0
				stage.grid[y+2][x] = 0
				o = 1
			end
		end
	elseif b == 3 then
		if o == 1 then
			if stage.grid[y-1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y+1][x-1] = 0
				stage.grid[y+1][x] = 0
				o = 2
			end
		elseif o == 2 then
			if x == 1 then	moveBlockRight() end
			if x > 1 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x+1] = 0
				o = 1
			end
		end
	elseif b == 4 then
		if o == 1 then
			if stage.grid[y-1][x+1] == 0 and stage.grid[y][x+1] == 0 then
				stage.grid[y-1][x+1] = b
				stage.grid[y][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x+1] = 0
				o = 2
			end
		elseif o == 2 then
			if x == 1 then moveBlockRight() end
			if x > 1 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x+1] = 0
				o = 1
			end
		end
	elseif b == 5 then
		if o == 1 then
			if stage.grid[y-1][x] == 0 and stage.grid[y+1][x] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y+1][x] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y][x-1] = 0
				stage.grid[y+1][x-1] = 0
				stage.grid[y][x+1] = 0
				o = 2
			end
		elseif o == 2 then
			if x == 1 then moveBlockRight() end
			if x > 1 and stage.grid[y][x-1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y-1][x+1] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y][x+1] = b
				stage.grid[y-1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x] = 0
				stage.grid[y+1][x+1] = 0
				o = 3
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y-1][x-1] == 0 and stage.grid[y-1][x] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y-1][x] = b
				stage.grid[y+1][x] = b
				stage.grid[y-1][x+1] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y][x+1] = 0
				o = 4
			end
		elseif o == 4 then
			if x == 10 then moveBlockLeft() end
			if x < 10 and stage.grid[y][x-1] == 0 and stage.grid[y+1][x-1] == 0 and stage.grid[y][x-1] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y][x+1] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x] = 0
				o = 1
			end
		end
	elseif b == 6 then
		if o == 1 then
			if stage.grid[y-1][x] == 0 and stage.grid[y-1][x+1] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y-1][x+1] = b
				stage.grid[y+1][x] = b
				stage.grid[y][x-1] = 0
				stage.grid[y][x+1] = 0
				stage.grid[y+1][x+1] = 0
				o = 2
			end
		elseif o == 2 then
			if x == 1 then moveBlockRight() end
			if stage.grid[y-1][x-1] == 0 and stage.grid[y][x-1] == 0 and stage.grid[y][x+1] == 0 then
				stage.grid[y-1][x-1] = b
				stage.grid[y][x-1] = b
				stage.grid[y][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y-1][x+1] = 0
				stage.grid[y+1][x] = 0
				o = 3
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y-1][x] == 0 and stage.grid[y+1][x-1] == 0 and stage.grid[y+1][x] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y+1][x-1] = b
				stage.grid[y+1][x] = b
				stage.grid[y-1][x-1] = 0
				stage.grid[y][x-1] = 0
				stage.grid[y][x+1] = 0
				o = 4
			end
		elseif o == 4 then
			if x == 10 then moveBlockLeft() end
			if stage.grid[y][x-1] == 0 and stage.grid[y][x+1] == 0 and stage.grid[y+1][x+1] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y][x+1] = b
				stage.grid[y+1][x+1] = b
				stage.grid[y-1][x] = 0
				stage.grid[y+1][x-1] = 0
				stage.grid[y+1][x] = 0
				o = 1
			end
		end
	elseif b == 7 then
		if o == 1 then
			if stage.grid[y-1][x] == 0 then
				stage.grid[y-1][x] = b
				stage.grid[y][x-1] = 0
				o = 2
			end
		elseif o == 2 then
			if x == 1 then moveBlockRight() end
			if stage.grid[y][x-1] == 0 then
				stage.grid[y][x-1] = b
				stage.grid[y+1][x] = 0
				o = 3
			end
		elseif o == 3 then
			if y < 21 and stage.grid[y+1][x] == 0 then
				stage.grid[y+1][x] = b
				stage.grid[y][x+1] = 0
				o = 4
			end
		elseif o == 4 then
			if x == 10 then moveBlockLeft() end
			if stage.grid[y][x+1] == 0 then
				stage.grid[y][x+1] = b
				stage.grid[y-1][x] = 0
				o = 1
			end
		end
	end
end

function checkRows()
	-- Completed rows are found, points are added, and the board is shifted down
	bonus = 0
	points = 0
	-- Check all rows.
	for r = 2, 21 do
		-- Check individual row.
		if stage.grid[r][1] > 0 and stage.grid[r][2] > 0 and  stage.grid[r][3] > 0 and  stage.grid[r][4] > 0
		and  stage.grid[r][5] > 0 and  stage.grid[r][6] > 0 and  stage.grid[r][7] > 0 and  stage.grid[r][8] > 0
		and  stage.grid[r][9] > 0 and  stage.grid[r][10] > 0 then
			-- Complete row found
			rowsComplete = rowsComplete + 1
			-- Level up sound.
			if(math.mod(rowsComplete,10) == 0) then
				love.audio.stop(levelUp)
				love.audio.play(levelUp)
			end
			-- Regular block break sound.
			love.audio.stop(blockBreak)
			love.audio.play(blockBreak)
			-- Counter bonus for number of rows completed.
			bonus = bonus + 1
			-- Points per row. Bottom rows are worth more.
			points = r-1
			-- Shift all blocks above the completed line down.
			for q = 0, r-2 do
				for c = 1, 10 do
					stage.grid[r-q][c] = stage.grid[r-q-1][c]
				end 
			end
		end
	end
	-- Points earned multiplied and adjusted by the bonus.
	-- Example completion at bottom row 20 = 20 points.
	if bonus == 4 then -- Bonus of 4 rows completed at once.
		score = score + points*5 -- Equals 100 points.
	else
		if bonus == 3 then -- Bonus of 3 rows completed at once.
			score = score + points*3 + math.ceil(points/2) -- Equals 70 points.
		else
			if bonus == 2 then -- Bonus of 2 rows completed at once.
				score = score + points*2 + math.ceil(points/4) -- Equals 45 points.
			else
				score = score + points -- Equals 20 points.
			end -- Top row equals a single point.
		end -- Player is rewarded for keeping more blocks off the board.
	end
	-- If the player score has changed then sort the scores.
	if(bonus > 0) then 
		sortHighscore()
	end
end

function drawTealSquare(x,y)
	-- Square block colors.
	love.graphics.setColor(64,128,123)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(191,255,251)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(128,255,246)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawRedSquare(x,y)
	-- Line block colors.
	love.graphics.setColor(128,26,26)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(255,153,153)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(255,50,50)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawBlueSquare(x,y)
	-- S shaped block colors.
	love.graphics.setColor(20,45,128)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(148,173,255)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(40,91,255)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawGreenSquare(x,y)
	-- Z shaped block colors.
	love.graphics.setColor(24,128,53)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(153,255,182)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(48,255,107)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawYellowSquare(x,y)
	-- L shaped block colors.
	love.graphics.setColor(153,150,57)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(255,252,176)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(255,250,95)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawPurpleSquare(x,y)
	-- J shaped block colors.
	love.graphics.setColor(118,33,128)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(246,161,255)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(236,66,255)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end

function drawGreySquare(x,y)
	-- T shaped block colors.
	love.graphics.setColor(94,94,94)
	love.graphics.rectangle("fill",x,y,11,11)
	love.graphics.setColor(229,229,229)
	love.graphics.rectangle("fill",x,y,9,9)
	love.graphics.setColor(184,184,184)
	love.graphics.rectangle("fill",x+2,y+2,7,7)
end
